@@ -1,7 +1,6 @@
 #' Metadata Extraction from NCBI/GISAID EpiCoV FASTA file
 #'
-#' This function retrieves metadata (ID, country, date) from the input FASTA file, with the source of, either 
-#' NCBI (with default FASTA header) or GISAID (with default FASTA header).
+#' This function retrieves metadata (ID, country, date) from the NCBI/GISAID EpiCoV FASTA file (default FASTA header expected).
 #'
 #' @param file_path path of fasta file
 #' @param source the source of fasta file, either "ncbi" or "GISAID"
@@ -33,13 +32,16 @@ extract_from_GISAID <- function(file_path){
             heads <- c(heads, line)
         }
     }
-    IDs <- c(); countrys <- c(); dates <- c()
+    IDs <- c(); countries <- c(); dates <- c()
     for(head in heads){
         ID <- str_extract(head, "EPI[^|]*"); IDs <- c(IDs, ID)
-        country <- strsplit(head, '/', fixed=T)[[1]][2]; countrys <- c(countrys, country)
+
+        country <- tryCatch(strsplit(head, '/', fixed=T)[[1]][2], error = function(e) "NA")
+        countries <- c(countries, country)
+
         date <- str_extract(head, "[0-9]{4}-[0-9]{2}-[0-9]{2}"); dates <- c(dates, date)
     }
-    return(data.frame('ID' = IDs, 'country' = countrys, 'date' = dates))
+    return(data.frame('ID' = IDs, 'country' = countries, 'date' = dates))
 }
 
 #' Extract metadata via fasta file from ncbi
@@ -57,7 +59,7 @@ extract_from_NCBI <- function(file_path){
             IDs <- c(IDs, ID)
         }
     }
-    countrys <- c(); dates <- c(); dropsample <- c(); keepsample <- c()
+    countries <- c(); dates <- c(); dropsample <- c(); keepsample <- c()
     for(ID in IDs){
         if(substr(ID,1,3) == 'pdb'){
             dropsample <- c(dropsample, ID)
@@ -66,16 +68,22 @@ extract_from_NCBI <- function(file_path){
         keepsample <- c(keepsample, ID)
         search_result <- entrez_search(db = "protein", term = ID, retmax = 1)
         accession <- search_result$ids[[1]]
+        
         info <- entrez_fetch(db = "protein", id = accession, rettype = "gb", retmode = "text")
-        info <- strsplit(info, '\n')[[1]]
+        info <- tryCatch(strsplit(info, '\n')[[1]], error = function(e) "")
         idx1 <- grep('/country=',  info); idx2 <- grep('/collection_date=',  info)
-        info1 <- info[idx1];               info2 <- info[idx2]
-        country <- strsplit(info1, '\\"')[[1]][2]
-        if(grepl(':', country)){country <- strsplit(country,':')[[1]][1]}
-        date <- strsplit(info2, '\\"')[[1]][2]
-        countrys <- c(countrys, country); dates <- c(dates, date)
+        info1 <- info[idx1]; info2 <- info[idx2]
+
+        country <- tryCatch(strsplit(info1, '\\"')[[1]][2], error = function(e) "NA")
+        if (grepl(':', country)) {
+            country <- strsplit(country,':')[[1]][1]
+        }
+
+        date <- tryCatch(strsplit(info2, '\\"')[[1]][2], error = function(e) "NA")
+        
+        countries <- c(countries, country); dates <- c(dates, date)
     }
-    tmp <- data.frame('ID' = keepsample, 'country' = countrys, 'date' = dates)
+    tmp <- data.frame('ID' = keepsample, 'country' = countries, 'date' = dates)
 
     for(i in 1:nrow(tmp)){
         if(!is.na(as.Date(tmp$date[i],format='%Y-%m-%d'))){
@@ -86,8 +94,8 @@ extract_from_NCBI <- function(file_path){
             dropsample <- c(dropsample, tmp$ID[i])
         }
     }
-    wraminfo <- paste(c(dropsample, 'did not provide a clear date, so it is excluded.'), collapse = " ")
-    warning(wraminfo)
+    warninfo <- paste(c('\nExcluded records:\n',dropsample, '\nCountry/date (%Y-%m-%d OR %d-%B-%Y format) are not provided.'), collapse = " ")
+    warning(warninfo)
     tmp <- tmp[! tmp$ID %in% dropsample, ]
     return(tmp)
 }
